# Category = Internet

=begin comment

internet_quakes.pl
 1.2 Switched from get_url to get_finger, periodic updates, and 
     %Save variable to remember old earthquakes so only new ones 
     are announced by David Norwood - 2/28/2001
 1.1 Distance thresholds by David Norwood <judapeno@gte.net>
     and other modifications by Tim Doyle - 2/18/2001
 1.0 Original version by Tim Doyle <tim@greenscourt.com>
     using news_yahoo.pl as an example - 2/15/2001

This script checks the USGS National Earthquake Information Center
via finger to get the last twenty one earthquakes that have occurred
in the world and presents those that were at least the minimum
magnitude(s) specified.

Possible future enhancements:
 - Add intelligent date/time information (i.e. 3 hours ago, yesterday at 5pm)

If you want to customize your magnitude thresholds, set a variable
like this in your ini file:

  Earthquake_Magnitudes = 99999 5.5 3000 3.5 100 0

You can also set a variable to limit the number of quakes to display.
To show only the two most recent quakes, regardless of magnitude, set
the following variables in your ini file:

  Earthquake_Magnitudes = 99999 0
  Earthquake_Count = 2


Note: If you live in the western hemisphere, this program needs your
longitude to be negative. This is a change to mh that has been
suggested but not yet accepted. It is unknown how this change might
affect other scripts that are expecting positive values. Until this
change has taken place, you can set the following variable in your
ini file to do the conversion in this program:

  Earthquake_Western = 1

=cut

# Default Magnitude Thresholds
my %Magnitude_thresholds = (
    99999,  5.5,     # show anything anywhere over 5.5
    500,    3.5,     # show anything within 500 miles over 3.5
    100,    0,       # show anything within 100 miles any size
);

if ($config_parms{Earthquake_Magnitudes}) {
  %Magnitude_thresholds = split ' ', $config_parms{Earthquake_Magnitudes};
}

# Maximum number of quakes to show
my $Earthquake_Count = 10;

if ($config_parms{Earthquake_Count}) {
  $Earthquake_Count = $config_parms{Earthquake_Count};
}

# Convert longitude for western hemisphere
my $MyLongitude = $config_parms{longitude};

if ($config_parms{Earthquake_Western}) {
  $MyLongitude = -1 * abs($MyLongitude);
}


my $f_earthquakes_finger = "$config_parms{data_dir}/web/earthquakes.finger";

$p_earthquakes = new Process_Item("get_finger quake\@gldfs.cr.usgs.gov $f_earthquakes_finger");

$v_earthquakes =  new  Voice_Cmd('[Get,Show,Read,Clear] recent earthquakes');
$v_earthquakes -> set_info('Display recent earthquake information');
$v_earthquakes -> set_authority('anyone');

$state = said $v_earthquakes;

if ( $state eq 'Get' or $New_Hour) {
  if (&net_connect_check) {
    print_log "Checking for recent earthquakes ...";
    speak("Checking for recent earthquakes, please hold on.");

    # Use start instead of run so we can detect when it is done
    start $p_earthquakes;
  }
}


if ( $state eq 'Show' ) {
  my $text = "Here are the recent earthquakes meeting your specified magnitudes: \n";
  $text .= $Save{quakes};
  $text =~ s/\t/\n/g;
  display $text;
}

if ( $state eq 'Clear' ) {
  print_log "Clearing recent earthquakes ...";
  $Save{quakes} = "";
}

if ( $state eq 'Read' ) {
  my $text = "Here are the recent earthquakes meeting your specified magnitudes: \n";
  speak $text . $Save{quakes};
}

if (done_now $p_earthquakes) {
  for (file_read "$f_earthquakes_finger") {
    chop;

    if (my ($qdate, $qtime, $qlatd, $qnoso, $qlong, $qeawe, $qdept, $qmagn, $qqual, $qloca) =
        $_ =~ m!^(\S+)\s+(\S+)\s+(\S+)([NS])\s+(\S+)([EW])\s+(\S+)\s+(\S+)M\s+(\S)?\s+(.+)! ) {

      $qlatd *= -1 if ( $qnoso eq "S" );
      $qlong *= -1 if ( $qeawe eq "W" );
      my $distance = sprintf "%d", calc_distance($config_parms{latitude},
        $MyLongitude, $qlatd, $qlong) + .5;
      for (keys %Magnitude_thresholds) {
        if ( $distance <= $_ and $qmagn >= $Magnitude_thresholds{$_}) {
          my $quake = "At $qtime on $qdate a magnitude $qmagn earthquake occurred $distance miles away ";

          if ( !($qloca =~ m/^(north\s|south\s|east\s|west\s|NE\s|NW\s|SE\s|SW\s|off\s|near\s|northeast\s|northwest\s|southeast\s|wouthwest\s).*/i)) {
            $quake .= "in ";
          }
          $quake .= "$qloca.\t";
          if ($Save{quakes} !~ /$quake/) {
            $Save{quakes} = $quake . $Save{quakes};
            $Save{quakes} =~ s/^(([^\t]*\t){1,$Earthquake_Count}).*/$1/;
            speak $quake;
          }
          last;
        }
      }
    }
  }
}

use Math::Trig;

sub calc_distance {
    my ($lat1, $lon1, $lat2, $lon2) = @_;
    my ($c, $d);
    $c = 57.3; # radian conversion factor

    $lat1 /= $c;
    $lat2 /= $c;
    $lon1 /= $c;
    $lon2 /= $c;
    $d = 2*Math::Trig::asin(sqrt((sin(($lat1-$lat2)/2))**2 + 	cos($lat1)*cos($lat2)*(sin(($lon1-$lon2)/2))**2));

    return $d*(.5*7915.6*.86838);  # convert to miles and return
}
